import 'package:dartx/dartx.dart';
import 'package:dartxx/dartxx.dart';
import 'package:intl/intl.dart';
import 'package:intl/locale.dart';
import 'package:logging/logging.dart';

import 'flexible_date.dart';

final _log = Logger("flexDateParser");

class FlexiDateParser {
  static Locale locale = Locale.parse("en-US");

  static FlexibleDate? tryFrom(input) {
    try {
      if (input == null) return null;
      return FlexiDateParser.from(input);
    } catch (e) {
      _log.finer("Date parse error: $e");
      return null;
    }
  }

  static FlexibleDate? from(dynamic input) {
    if (input == null) return null;
    if (input is FlexiDate) return input;
    if (input is DateTime) return FlexibleDate.ofDateTime(input);
    if (input is Map) return FlexiDate.fromMap(input);
    var inputString = "$input";
    if (inputString.isEmpty) {
      return null;
    } else {
      return parse(inputString);
    }
  }

  static FlexibleDate? tryParse(String input) {
    try {
      return parse(input);
    } catch (e) {
      _log.finer("Date parse error: $e");
      return null;
    }
  }

  static FlexibleDate parse(String toParse) {
    if (toParse.isEmpty) {
      throw Exception("Empty date cannot be parsed");
    }
    final parseAttempt = DateTime.tryParse(toParse.trim());
    if (parseAttempt != null) {
      return FlexibleDate.ofDateTime(parseAttempt);
    }

    final input = "$toParse";
    final tokenized = tokenizeString(input, splitAll: true);

    try {
      final parts = [
        for (var token in tokenized)
          if (token.isNumeric)
            token.toInt()
          else
            DateFormat.MMMM(locale.toLanguageTag()).parseLoose(token).month,
      ];

      final length = parts.length;
      switch (length) {
        case 3:
          if (parts[0] > 1000) {
            return FlexibleDate.of(
                year: parts[0], month: parts[1], day: parts[2]);
          } else if (parts[2] > 1000) {
            return FlexibleDate.of(
                year: parts[2], month: parts[0], day: parts[1]);
          } else {
            throw Exception("Invalid date - can't find year");
          }
        case 2:
          if (parts[0] > 1000) {
            return FlexibleDate.of(year: parts[0], month: parts[1]);
          } else if (parts[1] > 1000) {
            return FlexibleDate.of(year: parts[1], month: parts[0]);
          } else {
            return FlexibleDate.of(month: parts[0], day: parts[1]);
          }
        case 1:
          if (parts[0] < 1000)
            throw Exception("Unable to find year within date for '$toParse'");
          return FlexibleDate.of(year: parts[0]);
        default:
          throw Exception("Unable to extract date parameters from '$toParse'");
      }
    } catch (e) {
      _log.finer("Error parsing date: $e");
      return FlexibleDate.unparsed(input, "Error $e");
    }
  }

  static FlexibleDate? fromJson(json) {
    return FlexiDateParser.from(json);
  }
}
