import 'package:dartxx/dartxx.dart';
import 'package:flexidate/flexi_date_parser.dart';
import 'package:timezone/timezone.dart';

abstract class FlexibleDate {
  factory FlexibleDate.ofDateTime(DateTime dt) {
    return FlexiDate.fromDateTime(dt);
  }

  factory FlexibleDate.unparsed(String original, [String? message]) {
    return _UnparsedFlexiDate(original, message);
  }

  factory FlexibleDate.of({int? day, int? month = 1, int? year}) {
    return FlexiDate(day: day, month: month, year: year);
  }

  const FlexibleDate();

  dynamic toJson();

  Object? get source;

  int? get day => null;

  int? get month => null;

  int? get year => null;

  bool get isValid => false;
}

/// Represents a date that's couldn't be parsed.
class _UnparsedFlexiDate extends FlexibleDate {
  @override
  final String source;

  final String? message;

  const _UnparsedFlexiDate(this.source, [this.message]) : super();

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is _UnparsedFlexiDate && source == other.source;

  @override
  int get hashCode => source.hashCode;

  @override
  dynamic toJson() {
    return source;
  }

  @override
  String toString() {
    return source;
  }
}

extension FlexiDateExt on FlexibleDate {
  FlexiDate copy() {
    return FlexiDate(
      day: day,
      month: month,
      year: year,
      source: source,
    );
  }

  String? get error =>
      this is _UnparsedFlexiDate ? (this as _UnparsedFlexiDate).message : null;
}

/// A flexible container for date components that provides a robust parsing/building mechanism.  If the input type is known
/// to be a [String], [Map] or [DateTime], then use the corresponding constructors.
///
/// [FlexiDateParser.tryFrom] will / attempt to construct a [FlexibleDate] instance, and will return `null` if none could be constructed.
/// [FlexiDateParser.from] will / attempt to construct a [FlexibleDate] instance, and will raise an exception if unable to create a [FlexibleDate] instance
///
class FlexiDate implements FlexibleDate {
  @override
  int? day;

  @override
  int? month;

  @override
  int? year;

  @override
  Object? source;

  FlexiDate({this.day, this.month = 1, this.year, this.source});

  FlexiDate.fromDateTime(DateTime dateTime)
      : day = dateTime.day,
        month = dateTime.month,
        year = dateTime.year,
        // ignore: prefer_initializing_formals
        source = dateTime;

  factory FlexiDate.now() => FlexiDate.fromDateTime(DateTime.now());

  /// from a map, assuming keys [kday], [kmonth], [kyear]
  FlexiDate.fromMap(Map toParse)
      : day = _tryParseInt(toParse[kday]),
        month = _tryParseInt(toParse[kmonth]),
        year = _tryParseInt(toParse[kyear]),
        // ignore: prefer_initializing_formals
        source = toParse;

  @override
  dynamic toJson() {
    return "$this";
  }

  @override
  bool get isValid => true;

  Map<String, int?> toMap() {
    return {
      if (hasYear) kyear: year,
      if (hasMonth) kmonth: month,
      if (hasDay) kday: day,
    };
  }

  bool get isFullDate => year != null;

  bool get isFuture => isFullDate && toDateTime().isFuture;

  bool get hasMonth => month != null;

  bool get hasYear => year != null;

  bool get hasDay => day != null;

  @override
  String toString() => [year, month, day]
      .notNull()
      .map((part) => part < 10 ? "0$part" : "$part")
      .join("-");

  int millisecondsSinceEpoch([Location? location]) {
    return toDateTime(location).millisecondsSinceEpoch;
  }

  DateTime toDateTime([Location? location]) {
    if (location != null) {
      return TZDateTime(location, year ?? 1971, month ?? 1, day ?? 1);
    } else {
      return DateTime(year ?? 1971, month ?? 1, day ?? 1);
    }
  }

  FlexiDate withoutDay() => FlexiDate(day: null, month: month, year: year);

  FlexiDate withoutYear() => FlexiDate(day: day, month: month, year: null);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is FlexiDate &&
          day == other.day &&
          month == other.month &&
          year == other.year;

  @override
  int get hashCode => day.hashCode ^ month.hashCode ^ year.hashCode;
}

int? _tryParseInt(dyn) {
  if (dyn == null) return null;
  return int.tryParse("$dyn");
}

const kyear = 'year';
const kmonth = 'month';
const kday = 'day';

DateTime withoutTime(DateTime time) =>
    DateTime(time.year, time.month, time.day, 0, 0, 0, 0, 0);

bool hasTime(DateTime time) =>
    time.second != 0 ||
    time.minute != 0 ||
    time.hour != 0 ||
    time.millisecond != 0;

bool isFuture(DateTime? time) => time?.isAfter(DateTime.now()) == true;

bool isPast(DateTime? time) => time?.isBefore(DateTime.now()) == true;

extension DateComponentsComparisons on FlexiDate {
  bool isSameMonth(DateTime date) {
    return this.month == date.month && this.year == date.year;
  }
}
